# Personal Context (Auto-generated by mneia)

> Generated: {{ generated_at }}
> Documents: {{ stats.total_documents }} | Entities: {{ graph_stats.total_nodes }} | Relationships: {{ graph_stats.total_edges }}

## Overview

{% if overview %}
{{ overview }}
{% else %}
No overview available yet. Run `mneia connector sync` to ingest documents.
{% endif %}

## Key People

{% if people %}
{% for person in people %}
- **{{ person.name }}**{% if person.get('description') %}: {{ person.description }}{% endif %}

{% if summaries.get('person:' + person.name) %}
  {{ summaries['person:' + person.name] | indent(2) }}
{% endif %}
{% endfor %}
{% else %}
No people extracted yet.
{% endif %}

## Active Projects

{% if projects %}
{% for project in projects %}
- **{{ project.name }}**{% if project.get('description') %}: {{ project.description }}{% endif %}

{% endfor %}
{% else %}
No projects extracted yet.
{% endif %}

## Key Topics

{% if topics %}
{% for topic in topics %}
- **{{ topic.name }}**{% if topic.get('description') %}: {{ topic.description }}{% endif %}

{% endfor %}
{% else %}
No topics extracted yet.
{% endif %}

## Recent Decisions

{% if decisions %}
{% for decision in decisions %}
- {{ decision.name }}{% if decision.get('description') %}: {{ decision.description }}{% endif %}

{% endfor %}
{% else %}
No decisions extracted yet.
{% endif %}

## Beliefs & Values

{% if beliefs %}
{% for belief in beliefs %}
- {{ belief.name }}{% if belief.get('description') %}: {{ belief.description }}{% endif %}

{% endfor %}
{% else %}
No beliefs extracted yet.
{% endif %}
